<?php

class Immerseus_Calendar_Admin {
	/**
	 * @var string
	 */
	private $plugin_name;

	/**
	 * @var string
	 */
	private $version;

	/**
	 * @var Immerseus_Calendar_Upcoming_Events_Mapper
	 */
	private $events_mapper;

	public function __construct( string $plugin_name, string $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;

		$this->events_mapper = new Immerseus_Calendar_Upcoming_Events_Mapper();
	}

	public function schedule_event_reminders() {
		if ( false === as_next_scheduled_action( 'immerseus_calendar_remind_about_events' ) ) {
			as_schedule_recurring_action(
				strtotime( 'now' ),
				MINUTE_IN_SECONDS,
				'immerseus_calendar_remind_about_events'
			);
		}
	}

	public function send_event_reminders() {
		if ( ! get_option( 'imm_calendar_enable_reminders' ) ) {
			return;
		}

		/** @var WP_User[] $users */
		$users = get_users();

		foreach ( $users as $user ) {
			$events = $this->events_mapper->map( $user->ID );

			if ( empty( $events ) ) {
				continue;
			}

			// Map and send email

			$subject = __( 'You have an upcoming event', 'immerseus-calendar' );

			$body = __( 'Hello', 'immerseus-calendar' ) . " {$user->display_name},\n\n";
			$body .= __( 'You have upcoming events:', 'immerseus-calendar' ) . "\n\n";

			foreach ( $events as $event ) {
				$body .= $event['title'] . ' - ' . $event['date'] . "\n";
			}

			wp_mail( $user->user_email, $subject, $body );
		}
	}

	// Register Custom Post Type
	public function setup_calendar_event_post_type() {
		$labels = [
			'name'                  => _x( 'Calendar Events', 'Post Type General Name', 'immerseus-calendar' ),
			'singular_name'         => _x( 'Calendar Event', 'Post Type Singular Name', 'immerseus-calendar' ),
			'menu_name'             => __( 'Calendar for LearnDash', 'immerseus-calendar' ),
			'name_admin_bar'        => __( 'Calendar for LearnDash', 'immerseus-calendar' ),
			'archives'              => __( 'Calendar Events', 'immerseus-calendar' ),
			'attributes'            => __( 'Calendar Event Attributes', 'immerseus-calendar' ),
			'parent_item_colon'     => __( '', 'immerseus-calendar' ),
			'all_items'             => __( 'All Events', 'immerseus-calendar' ),
			'add_new_item'          => __( 'Add New Calendar Event', 'immerseus-calendar' ),
			'add_new'               => __( 'Add New', 'immerseus-calendar' ),
			'new_item'              => __( 'New Calendar Event', 'immerseus-calendar' ),
			'edit_item'             => __( 'Edit Event', 'immerseus-calendar' ),
			'update_item'           => __( 'Update Event', 'immerseus-calendar' ),
			'view_item'             => __( 'View Event', 'immerseus-calendar' ),
			'view_items'            => __( 'View Event', 'immerseus-calendar' ),
			'search_items'          => __( 'Search Event', 'immerseus-calendar' ),
			'not_found'             => __( 'Not found', 'immerseus-calendar' ),
			'not_found_in_trash'    => __( 'Not found in Trash', 'immerseus-calendar' ),
			'featured_image'        => __( 'Featured Image', 'immerseus-calendar' ),
			'set_featured_image'    => __( 'Set featured image', 'immerseus-calendar' ),
			'remove_featured_image' => __( 'Remove featured image', 'immerseus-calendar' ),
			'use_featured_image'    => __( 'Use as featured image', 'immerseus-calendar' ),
			'insert_into_item'      => __( 'Insert into item', 'immerseus-calendar' ),
			'uploaded_to_this_item' => __( 'Uploaded to this item', 'immerseus-calendar' ),
			'items_list'            => __( 'Items list', 'immerseus-calendar' ),
			'items_list_navigation' => __( 'Items list navigation', 'immerseus-calendar' ),
			'filter_items_list'     => __( 'Filter items list', 'immerseus-calendar' ),
		];

		$args = [
			'label'               => __( 'Calendar Event', 'immerseus-calendar' ),
			'labels'              => $labels,
			'supports'            => [ 'title', 'author' ],
			'hierarchical'        => false,
			'public'              => true,
			'show_ui'             => true,
			'show_in_menu'        => true,
			'menu_position'       => 25,
			'menu_icon'           => 'dashicons-calendar-alt',
			'show_in_admin_bar'   => false,
			'show_in_nav_menus'   => true,
			'can_export'          => true,
			'has_archive'         => false,
			'exclude_from_search' => true,
			'publicly_queryable'  => false,
			'rewrite'             => false,
			'capabilities'        => [
				"edit_post"              => "edit_imm_calendar_event",
				"read_post"              => "read_imm_calendar_event",
				"delete_post"            => "delete_imm_calendar_event",
				"edit_posts"             => "edit_imm_calendar_events",
				"edit_others_posts"      => "edit_others_imm_calendar_events",
				"delete_posts"           => "delete_imm_calendar_events",
				"publish_posts"          => "publish_imm_calendar_events",
				"read_private_posts"     => "read_private_imm_calendar_events",
				"read"                   => "read",
				"delete_private_posts"   => "delete_private_imm_calendar_events",
				"delete_published_posts" => "delete_published_imm_calendar_events",
				"delete_others_posts"    => "delete_others_imm_calendar_events",
				"edit_private_posts"     => "edit_private_imm_calendar_events",
				"edit_published_posts"   => "edit_published_imm_calendar_events",
				"create_posts"           => "edit_imm_calendar_events",
			],
//			'capability_type'     => 'imm_calendar_event',
//			'map_meta_cap'        => false,
			'show_in_rest'        => true,
		];

		register_post_type( 'imm_calendar_event', $args );
	}

	public function add_menu() {
		add_submenu_page(
			'learndash-lms',
			_x( 'Calendar Events', 'Post Type General Name', 'immerseus-calendar' ),
			_x( 'Calendar Events', 'Post Type General Name', 'immerseus-calendar' ),
			'edit_imm_calendar_events',
			'edit.php?post_type=imm_calendar_event'
		);

		add_submenu_page(
			'learndash-lms',
			__( 'Calendar Settings', 'immerseus-calendar' ),
			__( 'Calendar Settings', 'immerseus-calendar' ),
			'manage_options',
			'imm-calendar-settings',
			[ $this, 'settings' ]
		);

		remove_menu_page( 'edit.php?post_type=imm_calendar_event' );
	}

	public function limit_calendar_events_visibility( $query ) {
		if ( ! is_admin() or $query->query['post_type'] !== 'imm_calendar_event' ) {
			return;
		}

		if ( ! isset( $GLOBALS['current_screen'] ) and ! is_customize_preview() ) {
			// AJAX calls from frontend, excludes customize preview screen
			return;
		}

        $cur_user_id = get_current_user_id();

		if ( learndash_is_admin_user() ) {
	        $query->set( 'post_status', 'all' );
        } elseif ( learndash_is_group_leader_user() ) {
			$query->set( 'author', $cur_user_id );
		}

		$query->set( 'post_status', [ 'publish', 'private' ] );

//		$cur_user_id = get_current_user_id();
//
//		if ( learndash_is_admin_user() ) {
//			$group_leader_ids   = get_users( [
//				'role'   => 'group_leader',
//				'fields' => [ 'ID' ],
//			] );
//			$group_leader_ids   = wp_list_pluck( $group_leader_ids, 'ID' );
//			$group_leader_ids[] = $cur_user_id;
//			$query->set( 'author__in', $group_leader_ids );
//		} elseif ( learndash_is_group_leader_user() ) {
//			$query->set( 'author', $cur_user_id );
//		}
	}

	public function init_settings() {
		$general   = 'imm-calendar-settings';
		$reminders = 'imm-calendar-settings-reminders';

		// Register settings

		register_setting( $general, 'imm_calendar_theme' );
		register_setting( $general, 'imm_calendar_show_for_guests' );
		register_setting( $general, 'imm_calendar_enable_reminders' );
		register_setting( $general, 'imm_calendar_remind_minutes_before' );

		// General

		add_settings_section( $general, __( 'General', 'immerseus-calendar' ), null, $general );

		add_settings_field(
			'imm_calendar_theme',
			__( 'Calendar Theme', 'immerseus-calendar' ),
			function () {
				$value = get_option( 'imm_calendar_theme', 'default' );

				echo sprintf(
					'
		            <select name="imm_calendar_theme" style="width: 175px;">
		                <option value="default" %s>%s</option>
		                <option value="dark" %s>%s</option>
		            </select>
		            ',
					selected( $value, 'default', false ),
					__( 'Default', 'immerseus-calendar' ),
					selected( $value, 'dark', false ),
					__( 'Dark', 'immerseus-calendar' )
				);
			},
			$general,
			$general
		);

		add_settings_field(
			'imm_calendar_show_for_guests',
			__( 'Visible for not logged in users', 'immerseus-calendar' ),
			function () {
				$value = get_option( 'imm_calendar_show_for_guests', 0 );

				echo sprintf(
					'
                        <select name="imm_calendar_show_for_guests" style="width: 175px;">
                            <option value="1" %s>%s</option>
                            <option value="0" %s>%s</option>
                        </select>
                    ',
					selected( $value, 1, false ),
					__( 'Yes', 'immerseus-calendar' ),
					selected( $value, 0, false ),
					__( 'No', 'immerseus-calendar' )
				);
			},
			$general,
			$general
		);

		// Reminders

		add_settings_section( $reminders, __( 'Reminders', 'immerseus-calendar' ), null, $general );

		add_settings_field(
			'imm_calendar_enable_reminders',
			__( 'Enable Reminders', 'immerseus-calendar' ),
			function () {
				$value = get_option( 'imm_calendar_enable_reminders', 0 );

				echo sprintf(
					'
                        <select name="imm_calendar_enable_reminders" style="width: 175px;">
                            <option value="1" %s>%s</option>
                            <option value="0" %s>%s</option>
                        </select>
                    ',
					selected( $value, 1, false ),
					__( 'Yes', 'immerseus-calendar' ),
					selected( $value, 0, false ),
					__( 'No', 'immerseus-calendar' )
				);
			},
			$general,
			$reminders
		);

		add_settings_field(
			'imm_calendar_remind_minutes_before',
			__( 'Remind about upcoming events in advance', 'immerseus-calendar' ),
			function () {
				$options = [];

				$map_label = function ( $initial_minutes ) {
					$hours   = (int) ( $initial_minutes / 60 );
					$minutes = $initial_minutes - ( $hours * 60 );

					$label = '';

					if ( $hours ) {
						$label .= sprintf( $hours > 1 ? '%d hours ' : '%d hour ', $hours );
					}
					if ( $minutes ) {
						$label .= sprintf( '%d minutes', $minutes );
					}

					return $label;
				};

				for ( $i = 15; $i <= 60 * 24; ) {
					$options[ $i ] = $map_label( $i );

					$i += 15;
				}

				$value = get_option( 'imm_calendar_remind_minutes_before', 60 * 24 );

				echo '<select name="imm_calendar_remind_minutes_before" id="imm-calendar-remind-minutes-before" style="width: 175px;">';
				foreach ( $options as $option_value => $option_label ) {
					echo sprintf(
						'<option value="%d" %s>%s</option>',
						$option_value,
						selected( $value, $option_value, false ),
						esc_html( $option_label )
					);
				}
				echo '</select>';
			},
			$general,
			$reminders
		);
	}

	public function settings() {
		?>
        <div class="wrap">
            <h1 class="ldpm-admin-header">
				<?php _e( 'Calendar Settings', 'immerseus-calendar' ) ?>
            </h1>

			<?php settings_errors(); ?>

            <div class="postbox" style="margin-top: 15px; padding: 0 25px;">
                <form method="post" action="options.php">
					<?php settings_fields( 'imm-calendar-settings' ); ?>

					<?php do_settings_sections( 'imm-calendar-settings' ); ?>

					<?php submit_button(); ?>
                </form>
            </div>
        </div>
		<?php
	}

	public function add_caps_to_group_leaders() {
		// Admin

		$administrator = get_role( 'administrator' );

		$administrator->add_cap( 'publish_imm_calendar_events' );
		$administrator->add_cap( 'edit_imm_calendar_event' );
		$administrator->add_cap( 'edit_imm_calendar_events' );
		$administrator->add_cap( 'edit_published_imm_calendar_events' );
		$administrator->add_cap( 'edit_others_imm_calendar_events' );
		$administrator->add_cap( 'delete_others_imm_calendar_events' );
		$administrator->add_cap( 'delete_published_imm_calendar_events' );
		$administrator->add_cap( 'delete_imm_calendar_events' );
		$administrator->add_cap( 'delete_imm_calendar_event' );

		// Group leader

		$group_leader = get_role( 'group_leader' );

		$group_leader->add_cap( 'create_posts' );
		$group_leader->add_cap( 'publish_imm_calendar_events' );
		$group_leader->add_cap( 'edit_imm_calendar_event' );
		$group_leader->add_cap( 'edit_imm_calendar_events' );
		$group_leader->add_cap( 'edit_published_imm_calendar_events' );
		$group_leader->add_cap( 'delete_published_imm_calendar_events' );
		$group_leader->add_cap( 'delete_imm_calendar_events' );
		$group_leader->add_cap( 'delete_imm_calendar_event' );

		$group_leader->add_cap( 'read_others_imm_calendar_events', false );
		$group_leader->add_cap( 'edit_others_imm_calendar_events', false );
		$group_leader->add_cap( 'delete_others_imm_calendar_events', false );
	}

	public function add_calendar_event_meta_boxes() {
		global $post;

		$cur_user_id = get_current_user_id();

		add_meta_box(
			'imm_calendar_event_settings',
			__( 'Event', 'immerseus-calendar' ),
			function () use ( $post, $cur_user_id ) {
				$fields = [
					[
						'label'    => __( 'Link', 'immerseus-calendar' ),
						'name'     => 'link',
						'class'    => '',
						'type'     => 'url',
						'required' => true,
					],
					[
						'label'         => __( 'All Day', 'immerseus-calendar' ),
						'name'          => 'all_day',
						'class'         => 'imm-event-all-day-field',
						'type'          => 'checkbox',
						'default_value' => false,
					],
					[
						'label'    => __( 'Start Date', 'immerseus-calendar' ),
						'name'     => 'start_date',
						'class'    => 'imm-event-date-field imm-event-start-date-field',
						'type'     => 'date',
						'required' => true,
					],
					[
						'label' => __( 'End Date', 'immerseus-calendar' ),
						'name'  => 'end_date',
						'class' => 'imm-event-date-field imm-event-end-date-field',
						'type'  => 'date',
					],
				];

				echo wp_nonce_field( 'update_imm_calendar_event_meta', 'imm-calendar-event-meta-nonce' );

				$this->show_meta_fields( $fields, $post );
			},
			'imm_calendar_event',
			'normal',
			'high'
		);

		add_meta_box(
			'imm_calendar_event_styles',
			__( 'Styles', 'immerseus-calendar' ),
			function () use ( $post, $cur_user_id ) {
				$fields = [
					[
						'label'         => __( 'Background Color', 'immerseus-calendar' ),
						'name'          => 'background_color',
						'class'         => 'imm-event-color-field',
						'type'          => 'color',
						'default_value' => '#2270B1',
					],
					[
						'label'         => __( 'Text Color', 'immerseus-calendar' ),
						'name'          => 'text_color',
						'class'         => 'imm-event-color-field',
						'type'          => 'color',
						'default_value' => '#FFFFFF',
					],
				];

				$this->show_meta_fields( $fields, $post );
			},
			'imm_calendar_event',
			'normal',
			'high'
		);

		add_meta_box(
			'imm_calendar_event_visibility',
			__( 'Visibility', 'immerseus-calendar' ),
			function () use ( $post, $cur_user_id ) {
				$fields = [
					[
						'label'         => __( 'Show For', 'immerseus-calendar' ),
						'name'          => 'limited_to_type',
						'class'         => 'imm-event-select-field imm-event-limited-to-type-field',
						'type'          => 'select',
						'default_value' => 'users',
						'multiple'      => false,
						'options'       => [
							'all'               => __( 'All Users', 'immerseus-calendar' ),
							'users'             => __( 'Specific Users', 'immerseus-calendar' ),
							'learndash_groups'  => __( 'LearnDash Groups', 'immerseus-calendar' ),
							'learndash_courses' => __( 'LearnDash Courses', 'immerseus-calendar' ),
						],
					],
					[
						'label'         => __( 'Specific Users', 'immerseus-calendar' ),
						'name'          => 'limited_to_user_ids',
						'class'         => 'imm-event-select-field imm-event-limited-to-ids-field imm-event-users-field',
						'type'          => 'select',
						'default_value' => 'all',
						'multiple'      => true,
						'options'       => wp_list_pluck(
							get_users( [
								'include' => learndash_is_admin_user( $cur_user_id ) ? [] : learndash_get_group_leader_groups_users( $cur_user_id ),
								'fields'  => [ 'ID', 'user_nicename' ],
							] ),
							'user_nicename',
							'ID'
						),
					],
					[
						'label'         => __( 'LearnDash Groups', 'immerseus-calendar' ),
						'name'          => 'limited_to_group_ids',
						'class'         => 'imm-event-select-field imm-event-limited-to-ids-field imm-event-groups-field',
						'type'          => 'select',
						'default_value' => '',
						'multiple'      => true,
						'options'       => wp_list_pluck(
							learndash_is_admin_user( $cur_user_id )
								? learndash_get_groups()
								: get_posts( [
								'post_type' => 'groups',
								'post__in'  => learndash_get_administrators_group_ids( $cur_user_id ),
							] ),
							'post_title',
							'ID'
						),
					],
					[
						'label'         => __( 'LearnDash Courses', 'immerseus-calendar' ),
						'name'          => 'limited_to_course_ids',
						'class'         => 'imm-event-select-field imm-event-limited-to-ids-field imm-event-courses-field',
						'type'          => 'select',
						'default_value' => '',
						'multiple'      => true,
						'options'       => wp_list_pluck(
							get_posts( [
								'post_type' => 'sfwd-courses',
								'post__in'  => learndash_get_user_course_access_list( $cur_user_id ),
							] ),
							'post_title',
							'ID'
						),
					],
				];

				$this->show_meta_fields( $fields, $post );
			},
			'imm_calendar_event',
			'normal',
			'high'
		);
	}

	public function save_calendar_event_post_type_meta( $post_id ) {
		if ( ! isset( $_REQUEST['imm-calendar-event-meta-nonce'] ) or ! wp_verify_nonce( $_REQUEST['imm-calendar-event-meta-nonce'], 'update_imm_calendar_event_meta' ) ) {
			return;
		}

		$is_all_day = isset( $_REQUEST['imm_calendar_event_all_day'] );

		// Main

		update_post_meta( $post_id, 'imm_calendar_event_link', wp_http_validate_url( $_REQUEST['imm_calendar_event_link'] ) );

		update_post_meta( $post_id, 'imm_calendar_event_all_day', $is_all_day );

		update_post_meta( $post_id, 'imm_calendar_event_start_date', get_gmt_from_date( sanitize_text_field( $_REQUEST['imm_calendar_event_start_date'] ) ) );

		if ( $is_all_day ) {
			delete_post_meta( $post_id, 'imm_calendar_event_end_date' );
		} else {
			update_post_meta( $post_id, 'imm_calendar_event_end_date', get_gmt_from_date( sanitize_text_field( $_REQUEST['imm_calendar_event_end_date'] ) ) );
		}

		// Styles

		update_post_meta( $post_id, 'imm_calendar_event_background_color', sanitize_hex_color( $_REQUEST['imm_calendar_event_background_color'] ?: '#2270B1' ) );
		update_post_meta( $post_id, 'imm_calendar_event_text_color', sanitize_hex_color( $_REQUEST['imm_calendar_event_text_color'] ?: '#FFFFFF' ) );

		// Visibility

		$imm_calendar_event_limited_to_type = sanitize_text_field( $_REQUEST['imm_calendar_event_limited_to_type'] );

		update_post_meta( $post_id, 'imm_calendar_event_limited_to_type', $imm_calendar_event_limited_to_type );

		if ( $imm_calendar_event_limited_to_type === 'all' ) {
			delete_post_meta( $post_id, 'imm_calendar_event_limited_to_ids' );
		}
		if ( $imm_calendar_event_limited_to_type === 'users' ) {
			update_post_meta( $post_id, 'imm_calendar_event_limited_to_ids', $_REQUEST['imm_calendar_event_limited_to_user_ids'] );
		} elseif ( $imm_calendar_event_limited_to_type === 'learndash_groups' ) {
			update_post_meta( $post_id, 'imm_calendar_event_limited_to_ids', $_REQUEST['imm_calendar_event_limited_to_group_ids'] );
		} elseif ( $imm_calendar_event_limited_to_type === 'learndash_courses' ) {
			update_post_meta( $post_id, 'imm_calendar_event_limited_to_ids', $_REQUEST['imm_calendar_event_limited_to_course_ids'] );
		}
	}

	public function add_course_and_quiz_meta_boxes() {
		foreach ( [ 'sfwd-courses', 'sfwd-quiz' ] as $post_type ) {
			add_meta_box(
				'imm_calendar_due_date',
				__( 'Calendar', 'immerseus-calendar' ),
				function ( $post ) {
					$due_date_value = (string) get_post_meta( $post->ID, 'imm_calendar_due_date', true );
					if ( $due_date_value ) {
						$due_date_value = get_date_from_gmt( $due_date_value, 'Y/m/d H:i' );
					}
					?>
					<?php wp_nonce_field( 'update_field_imm_calendar_custom_meta_nonce', 'imm_calendar_custom_meta_nonce' ); ?>

                    <div class="components-base-control__field" style="margin-top: 10px;">
                        <label for="imm_calendar_due_date" class="components-form-token-field__label"
                               style="display: inline-block; margin-bottom: 8px;">
							<?php _e( 'Due Date', 'immerseus-calendar' ); ?>
                        </label>

                        <input name="imm_calendar_due_date" id="imm_calendar_due_date"
                               class="components-text-control__input imm-calendar-meta-date-field"
                               value="<?php esc_attr_e( $due_date_value ); ?>" />
                    </div>
					<?php
				},
				$post_type,
				'side'
			);
		}
	}

	public function save_course_and_quiz_meta() {
		if ( ! wp_verify_nonce( $_POST['imm_calendar_custom_meta_nonce'], 'update_field_imm_calendar_custom_meta_nonce' ) ) {
			return;
		}

		if ( ! in_array( $_POST['post_type'], [ 'sfwd-courses', 'sfwd-quiz' ] ) ) {
			return;
		}

		if ( ! empty( $_POST['imm_calendar_due_date'] ) ) {
			update_post_meta(
				intval( $_POST['post_ID'] ),
				'imm_calendar_due_date',
				get_gmt_from_date( sanitize_text_field( $_POST['imm_calendar_due_date'] ) )
			);
		}
	}

	public function enqueue_styles() {
		wp_enqueue_style(
			'toggle-switch',
			plugin_dir_url( __FILE__ ) . 'css/toggle-switch.css',
			[],
			$this->version
		);
		wp_enqueue_style(
			'select2',
			plugin_dir_url( __FILE__ ) . 'css/select2.min.css',
			[],
			$this->version
		);

		wp_enqueue_style(
			'jquery-datetimepicker',
			plugin_dir_url( __FILE__ ) . 'css/jquery.datetimepicker.min.css',
			[],
			$this->version
		);

		wp_enqueue_style( 'wp-color-picker' );

		wp_enqueue_style(
			$this->plugin_name,
			plugin_dir_url( __FILE__ ) . 'css/immerseus-calendar-admin.css',
			[],
			$this->version
		);
	}

	public function enqueue_scripts() {
		wp_enqueue_script(
			'select2',
			plugin_dir_url( __FILE__ ) . 'js/select2.min.js',
			[],
			$this->version
		);

		wp_enqueue_script(
			'jquery-datetimepicker',
			plugin_dir_url( __FILE__ ) . 'js/jquery.datetimepicker.full.min.js',
			[ 'jquery' ],
			$this->version
		);

		wp_enqueue_script(
			$this->plugin_name,
			plugin_dir_url( __FILE__ ) . 'js/immerseus-calendar-admin.js',
			[ 'jquery', 'wp-color-picker' ],
			$this->version
		);
	}

	protected function show_meta_fields( array $fields, $post ) {
		$prefix = 'imm_calendar_event_';

		echo '<table class="form-table">';

		foreach ( $fields as $field ) {
			$field['name'] = $prefix . $field['name'];

			$meta_value_name = $field['name'];

			if ( in_array( $field['name'], [
				$prefix . 'limited_to_user_ids',
				$prefix . 'limited_to_group_ids',
				$prefix . 'limited_to_course_ids',
			] ) ) {
				$meta_value_name = $prefix . 'limited_to_ids';
			}

			$meta_value = get_post_meta( $post->ID, $meta_value_name, true );

			echo '<tr>';

			echo '<th>' .
			     sprintf( '<label for="%s">%s</label>', $field['name'], $field['label'] ) .
			     '</th>';

			echo '<td>';

			switch ( $field['type'] ) {
				case 'text':
					echo sprintf(
						'<input type="text" name="%s" class="%s" value="%s" style="width: 100%%;" />',
						esc_attr( $field['name'] ),
						esc_attr( $field['class'] ),
						esc_attr( $meta_value )
					);
					break;
				case 'url':
					echo sprintf(
						'<input type="url" name="%s" class="%s" value="%s" style="width: 100%%;" />',
						esc_attr( $field['name'] ),
						esc_attr( $field['class'] ),
						esc_attr( $meta_value )
					);
					break;
				case 'date':
					echo sprintf(
						'<input type="text" name="%s" class="%s" value="%s" style="width: 100%%;" />',
						esc_attr( $field['name'] ),
						esc_attr( $field['class'] ),
						esc_attr( get_date_from_gmt( $meta_value, 'Y/m/d H:i' ) )
					);
					break;
				case 'color':
					echo sprintf(
						'<input type="text" name="%s" class="%s" value="%s" data-default-color="%s" />',
						esc_attr( $field['name'] ),
						esc_attr( $field['class'] ),
						esc_attr( $meta_value ?: $field['default_value'] ),
						esc_attr( $field['default_value'] )
					);
					break;
				case 'checkbox':
					echo sprintf(
						'<label class="toggle-switch">
									<input type="checkbox" name="%s" id="toggle-switch-input" class="toggle-switch-input %s" %s />
									<label for="toggle-switch-input" class="toggle-switch-label"></label>
								</label>',
						esc_attr( $field['name'] ),
						esc_attr( $field['class'] ),
						( $meta_value ?: $field['default_value'] ) ? 'checked="checked"' : ''
					);
					break;
				case 'select':
					echo sprintf(
						'<select name="%s" class="%s" %s style="width: 100%%;" />',
						esc_attr( $field['name'] ) . ( $field['multiple'] ? '[]' : '' ),
						esc_attr( $field['class'] ),
						$field['multiple'] ? 'multiple' : ''
					);

					foreach ( $field['options'] as $option_value => $option_label ) {
						echo sprintf(
							'<option value="%s" %s />%s</option>',
							esc_attr( $option_value ),
							in_array( $option_value, (array) ( $meta_value ?: $field['default_value'] ) ) ? ' selected="selected"' : '',
							esc_html( $option_label )
						);
					}
					echo '</select>';
					break;
			}

			echo '</td>';

			echo '</tr>';
		}

		echo '</table>';
	}
}
