<?php

/**
 * The plugin bootstrap file
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://honorswp.com
 * @since             2.0.0
 * @package           Immerseus_Calendar
 * @wordpress-plugin
 * Plugin Name:       Calendar for LearnDash
 * Plugin URI:        https://honorswp.com/
 * Description:       Calendar for LearnDash allows you to add calendar events for students and staff!
 * Version:           2.2.0
 * Author:            HonorsWP
 * Author URI:        https://honorswp.com
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       immerseus-calendar
 * Domain Path:       /languages
 */

// namespace EA_CALENDAR_LD;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

require_once __DIR__ . '/vendor/autoload.php';

use EA\Licensing\License;

/** Constants */
define( 'IMMERSEUS_CALENDAR_VERSION', '2.2.0' );
define( 'IMMERSEUS_CALENDAR_PATH', plugin_dir_path( __FILE__ ) );
define( 'IMMERSEUS_CALENDAR_URL', plugin_dir_url( __FILE__ ) );

/**
 * Licensing.
 */
new License(
	__( 'Calendar for LearnDash', 'immerseus-calendar' ),
	6644,
	'calendar-for-learndash',
	__FILE__,
	IMMERSEUS_CALENDAR_VERSION
);

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-immerseus-calendar-activator.php
 */
function activate_immerseus_calendar() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-immerseus-calendar-activator.php';
	Immerseus_Calendar_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-immerseus-calendar-deactivator.php
 */
function deactivate_immerseus_calendar() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-immerseus-calendar-deactivator.php';
	Immerseus_Calendar_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_immerseus_calendar' );
register_deactivation_hook( __FILE__, 'deactivate_immerseus_calendar' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-immerseus-calendar.php';

require_once plugin_dir_path( __FILE__ ) . 'vendor/woocommerce/action-scheduler/action-scheduler.php';

// require_once dirname( __FILE__ ) . '/vendor/immerseus/immerseus-upgrader/src/index.php';

/**
 * Begins execution of the plugin.
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_immerseus_calendar() {
	$plugin = new Immerseus_Calendar();
	$plugin->run();
}

run_immerseus_calendar();
