<?php

class Immerseus_Calendar_Event_DTO {
	const DEFAULT_BACKGROUND_COLOR = '#2270B1';

	const DEFAULT_TEXT_COLOR = '#FFFFFF';

	protected $id;

	protected $title;

	protected $start;

	protected $end;

	protected $all_day = false;

	protected $background_color = self::DEFAULT_BACKGROUND_COLOR;

	protected $border_color = self::DEFAULT_BACKGROUND_COLOR;

	protected $text_color = self::DEFAULT_TEXT_COLOR;

	protected $extended_props = [
		'url'      => '',
		'isAuthor' => false,
	];

	public function __construct( int $id, string $title, string $start ) {
		$this->id    = $id;
		$this->title = $title;
		$this->start = $start;

		do_action( 'create_immerseus_calendar_event', $id, $this );
	}

	public function set_end( string $end ) {
		$this->end = $end;
	}

	public function set_all_day( bool $all_day ) {
		$this->all_day = $all_day;
	}

	public function set_background_color( string $color ) {
		$value = sanitize_hex_color( $color ) ?: self::DEFAULT_BACKGROUND_COLOR;

		$this->background_color = $value;
		$this->border_color     = $value;
	}

	public function set_text_color( string $color ) {
		$this->text_color = sanitize_hex_color( $color ) ?: self::DEFAULT_TEXT_COLOR;
	}

	public function set_extended_props( $value, string $param, bool $ignore_key_existence = false ) {
		if ( ! $ignore_key_existence AND ! array_key_exists( $param, $this->extended_props ) ) {
			return;
		}

		$this->extended_props[ $param ] = $value;
	}

	public function to_array() {
		$result = [];

		foreach ( get_object_vars( $this ) as $key => $value ) {
			if ( $key === 'end' and is_null( $value ) ) {
				continue;
			}

			$key = lcfirst(
				str_replace( '_', '', ucwords( $key, '_' ) )
			);

			$result[ $key ] = $value;
		}

		return $result;
	}
}
