<?php

use Spatie\IcalendarGenerator\Components\Calendar;
use Spatie\IcalendarGenerator\Components\Event;

class Immerseus_Calendar_Exporter {
	const ACTION = 'feed';

	const SLUG = 'immerseus-calendar';

	/**
	 * @param array $headers
	 *
	 * @return array
	 */
	public function set_headers( $headers ) {
		if ( ! $this->is_calendar_feed_request() ) {
			return;
		}

		$headers['Content-Type'] = 'text/calendar; charset=UTF-8';

		return $headers;
	}

	public static function map_feed_url(): string {
		if ( ! is_user_logged_in() ) {
			return '';
		}

		$user_id    = get_current_user_id();
		$secret_key = static::map_user_secret_key( $user_id );

		return add_query_arg( [
			'action'     => static::ACTION,
			'user_id'    => $user_id,
			'secret_key' => $secret_key,
		], home_url( static::SLUG ) );
	}

	public static function map_user_secret_key( int $user_id ): string {
		$user = get_user_by( 'id', $user_id );

		return md5( $user->ID . $user->user_registered );
	}

	public function map_feed() {
		if ( ! $this->is_calendar_feed_request() ) {
			return;
		}

		$secret_key = sanitize_text_field( $_GET['secret_key'] ?? '' );
		$user_id    = intval( $_GET['user_id'] ?? 0 );

		if ( empty( $secret_key ) or empty( $user_id ) ) {
			return;
		}

		$user = get_user_by( 'id', $user_id );

		if ( ! $user or $this->map_user_secret_key( $user_id ) !== $secret_key ) {
			return;
		}

		$calendar_title = get_bloginfo( 'name' ) . ' ' . __( 'Calendar', 'immerseus-calendar' );

		$calendar = Calendar::create( $calendar_title );
		$calendar->event( $this->map_events( $user_id ) );

		exit( $calendar->get() );
	}

	private function map_events( int $user_id ): array {
		$events = [];

		$calendar_events_mapper = new Immerseus_Calendar_Events_Mapper();
		$wp_timezone            = wp_timezone();

		foreach ( $calendar_events_mapper->map( $user_id ) as $event_dto ) {
			$event = Event::create( $event_dto['title'] );

			$event->startsAt( new DateTime( $event_dto['start'], $wp_timezone ) );

			if ( isset( $event_dto['end'] ) ) {
				$event->endsAt( new DateTime( $event_dto['end'], $wp_timezone ) );
			}

			if ( $event_dto['allDay'] ) {
				$event->fullDay();
			}

			$events[] = $event;
		}

		return $events;
	}

	private function is_calendar_feed_request(): bool {
		global $wp;

		if ( self::SLUG !== $wp->request ) {
			return false;
		}

		return ( $_GET['action'] ?? '' === self::ACTION );
	}
}
