<?php

class Immerseus_Calendar_Upcoming_Events_Mapper extends Immerseus_Calendar_Events_Mapper {
	public function map( int $user_id ): array {
		$event_time_to_be_notified = $this->calculate_event_time_to_be_notified();

		return array_merge(
			$this->map_upcoming_lessons_for_students( $user_id, $event_time_to_be_notified ),
			$this->map_upcoming_calendar_events( $user_id, $event_time_to_be_notified ),
			$this->map_upcoming_courses_and_quizzes( $user_id, $event_time_to_be_notified )
		);
	}

	protected function map_upcoming_lessons_for_students( int $user_id, int $event_time_to_be_notified ): array {
		$lessons = $this->get_lessons_by_user( $user_id );

		if ( empty( $lessons ) ) {
			return [];
		}

		$events = [];

		foreach ( $lessons as $lesson_id => $lesson_title ) {
			$lesson_access_from_timestamp = ld_lesson_access_from( $lesson_id, $user_id );

			if ( $event_time_to_be_notified != $lesson_access_from_timestamp ) {
				continue;
			}

			$events[] = $this->map_event(
				$lesson_title . ' ' . __( 'becomes available', 'immerseus-calendar' ),
				date( 'Y-m-d H:i:s', $lesson_access_from_timestamp )
			);
		}

		return $events;
	}

	protected function map_upcoming_calendar_events( int $user_id, int $event_time_to_be_notified ): array {
		$query = $this->map_calendar_events_query( $user_id );

		if ( is_null( $query ) ) {
			return [];
		}

		// Setting meta query directly does affect the sql
		$query = new WP_Query( array_merge(
			$query->query_vars,
			[
				'meta_query' => [
					[
						'key'   => 'imm_calendar_event_start_date',
						'value' => date( 'Y-m-d H:i:s', $event_time_to_be_notified ),
					],
				],
			]
		) );

		$events = [];

		while ( $query->have_posts() ) {
			$query->the_post();

			$events[] = $this->map_event(
				get_the_title(),
				get_post_meta( get_the_ID(), 'imm_calendar_event_start_date', true )
			);
		}

		wp_reset_postdata();

		return $events;
	}

	protected function map_upcoming_courses_and_quizzes( int $user_id, int $event_time_to_be_notified ): array {
		$query = $this->map_courses_and_quizzes_query( $user_id );

		if ( is_null( $query ) ) {
			return [];
		}

		// Setting meta query directly does affect the sql
		$query = new WP_Query( array_merge(
			$query->query_vars,
			[
				'meta_query' => [
					[
						'key'   => 'imm_calendar_due_date',
						'value' => date( 'Y-m-d H:i:s', $event_time_to_be_notified ),
					],
				],
			]
		) );

		$events = [];

		while ( $query->have_posts() ) {
			$query->the_post();

			$events[] = $this->map_event(
				get_the_title() . ' ' . __( 'becomes unavailable', 'immerseus-calendar' ),
				get_post_meta( get_the_ID(), 'imm_calendar_due_date', true )
			);
		}

		wp_reset_postdata();

		return $events;
	}

	private function map_event( string $title, string $date ): array {
		return [
			'title' => $title,
			'date'  => get_date_from_gmt( $date, 'j F Y h:i A' ),
		];
	}

	private function calculate_event_time_to_be_notified() {
		$remind_in_minutes = get_option( 'imm_calendar_remind_minutes_before' );

		$time = time();
		$time -= $time % 60; // remove seconds

		return $time + $remind_in_minutes;
	}
}
