(function ($) {
    'use strict';

    window.current_event = null;

    function toggle_end_date_field_visibility() {
        if ($('#imm_event_form_all_day_input').is(':checked')) {
            $('.imm-event-form-end-date-field').hide();
        } else {
            $('.imm-event-form-end-date-field').show();
        }
    }

    $(function () {

        toggle_end_date_field_visibility();

        $('.imm-event-form-color-field input').wpColorPicker();

        $('.imm-event-form-date-field input').datetimepicker({
            theme: imm_calendar.theme,
        });

        $('#imm-event-form-all-day-field').on('click', function () {
            toggle_end_date_field_visibility();
        });

        // Custom Parsley validator

        window.Parsley.addValidator('requiredIfNotChecked', {
            validateString: function (value, requirement) {
                if (!jQuery(requirement).is(':checked')) {
                    return !!value;
                }

                return true;
            },
            messages: {
                en: 'This value is required.',
            },
            priority: 33,
        });

        const modal_options = {
            onClose: function (modal) {
                if (modal.id === 'event-form-modal') {
                    $('#event-form-modal .alert-success').hide();

                    $('#imm_event_form_post_id_input').val('');
                    $('#imm_event_form_title_input').val('');
                    $('#imm_event_form_link_input').val('');
                    $('#imm_event_form_all_day_input').prop('checked', false);
                    $('#imm_event_form_start_date_input').val('');
                    $('#imm_event_form_end_date_input').val('');
                    $('#imm_event_form_background_color_input').wpColorPicker('color', '#2270B1');
                    $('#imm_event_form_text_color_input').wpColorPicker('color', '#FFFFFF');
                    toggle_end_date_field_visibility();
                } else if (modal.id === 'event-view-modal') {
                    $('#event-view-modal .alert-success').hide();

                    $('#imm-event-author-actions').hide();
                }
            },
        };

        // Modal

        MicroModal.init(modal_options);

        // Event adding (with a validation)

        $('#imm-event-add').on('click', function () {
            $('#imm-event-edit-title').hide();
            $('#imm-event-add-title').show();

            MicroModal.close('event-view-modal', modal_options);

            MicroModal.show('event-form-modal', modal_options);

            return false;
        });

        $('#imm-event-form').parsley()
            .on('form:submit', function () {
                $.ajax({
                    type: 'POST',
                    dataType: 'json',
                    url: imm_calendar.ajaxurl,
                    data: {
                        action: 'imm_create_or_update_calendar_event',
                        nonce: imm_calendar.nonce,
                        form_data: $('#imm-event-form').serializeArray(),
                    },
                    success: function () {
                        calendar.refetchEvents();

                        $('#imm-event-form .alert-success').show();

                        setTimeout(function () {
                            MicroModal.close('event-form-modal', modal_options);
                        }, 3500);
                    },
                });

                return false;
            });

        // Event editing

        $('#imm-event-edit').on('click', function () {
            $('#imm-event-add-title').hide();
            $('#imm-event-edit-title').show();

            $('#imm_event_form_post_id_input').val(window.current_event.id);
            $('#imm_event_form_title_input').val(window.current_event.title);
            $('#imm_event_form_link_input').val(window.current_event.extendedProps.url);
            $('#imm_event_form_all_day_input').prop('checked', window.current_event.allDay);
            if (window.current_event.end) {
                $('#imm_event_form_start_date_input').val(
                    moment(window.current_event.start).format('YYYY/MM/DD HH:mm'),
                );
                $('#imm_event_form_end_date_input').val(
                    moment(window.current_event.end).format('YYYY/MM/DD HH:mm'),
                );
            } else {
                $('#imm_event_form_start_date_input').val(
                    moment(window.current_event.start).format('YYYY/MM/DD 00:00'),
                );
            }
            $('#imm_event_form_background_color_input').wpColorPicker('color', window.current_event.backgroundColor);
            $('#imm_event_form_text_color_input').wpColorPicker('color', window.current_event.textColor);
            toggle_end_date_field_visibility();

            $(document).triggerHandler('on_edit_learndash_calendar_event');

            MicroModal.close('event-view-modal', modal_options);

            MicroModal.show('event-form-modal', modal_options);

            return false;
        });

        // Event deleting

        $('#imm-event-delete').on('click', function () {
            $.ajax({
                type: 'POST',
                dataType: 'json',
                url: imm_calendar.ajaxurl,
                data: {
                    action: 'imm_delete_calendar_event',
                    nonce: imm_calendar.nonce,
                    post_id: window.current_event.id,
                },
                success: function () {
                    calendar.refetchEvents();

                    $('#event-view-modal .alert-success').show();

                    setTimeout(function () {
                        MicroModal.close('event-view-modal', modal_options);
                    }, 3500);
                },
            });

            return false;
        });

        // Calendar

        const calendar_el = document.getElementById('imm-full-calendar');
        const calendar = new FullCalendar.Calendar(calendar_el, {
            locale: imm_calendar.locale,
            headerToolbar: {
                left: 'prev,next today',
                center: 'title',
                right: 'dayGridMonth,timeGridWeek,timeGridDay',
            },
            eventSources: [
                {
                    url: imm_calendar.ajaxurl,
                    method: 'POST',
                    extraParams: {
                        action: 'imm_fetch_calendar_events',
                        nonce: imm_calendar.nonce,
                    },
                },
            ],
            eventClick: function (info) {
                window.current_event = info.event;

                $('#imm-event-author-actions')
                    .toggle(!!info.event.extendedProps.isAuthor);

                $('#event-view-modal .modal__title')
                    .text(info.event.title);

                $('#imm-event-link')
                    .attr('href', info.event.extendedProps.url)
                    .toggle(!!info.event.extendedProps.url);

                let modal_content = '';

                if (!!info.event.allDay) {
                    modal_content = moment(info.event.start).format('MMMM Do, YYYY');
                } else {
                    modal_content = moment(info.event.start).format('MMMM Do YYYY, HH:mm');

                    if (info.event.end) {
                        modal_content += ' - ';
                        modal_content += moment(info.event.end).format('MMMM Do YYYY, HH:mm');
                    }
                }

                $('#event-view-modal .modal__content')
                    .text(modal_content);

                $(document).triggerHandler('on_show_learndash_calendar_event');

                MicroModal.show('event-view-modal', modal_options);
            },
        });

        calendar.render();
    });
})(jQuery);
